// $Id$
/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.ui.combo;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Label;

import eu.etaxonomy.cdm.model.common.IDefinedTerm;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.cdm.model.reference.ReferenceType;
import eu.etaxonomy.taxeditor.preference.Resources;
import eu.etaxonomy.taxeditor.store.StoreUtil;
import eu.etaxonomy.taxeditor.ui.element.AbstractCdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.CdmFormFactory;
import eu.etaxonomy.taxeditor.ui.element.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.ui.element.ICdmFormElement;
import eu.etaxonomy.taxeditor.ui.element.IEnableableFormElement;
import eu.etaxonomy.taxeditor.ui.element.ISelectable;
import eu.etaxonomy.taxeditor.ui.element.LayoutConstants;

/**
 * <p>
 * Abstract AbstractEnumComboElement class.
 * </p>
 * 
 * @author n.hoffmann
 * @created Mar 16, 2010
 * @version 1.0
 */
public class EnumComboElement<T extends IDefinedTerm> extends
		AbstractCdmFormElement implements SelectionListener,
		IEnableableFormElement, ISelectable {

	private static CdmEnumDataHolder[] cdmEnumDataHolders = {
		new CdmEnumDataHolder<NomenclaturalCode>(){
			
			@Override
			public String getName() {
				return "Nomenclatural Code";
			}

			@Override
			public Collection<NomenclaturalCode> getElements() {
				return  Arrays.asList(NomenclaturalCode.values());
			}

			@Override
			public Class<NomenclaturalCode> getClazz() {
				return NomenclaturalCode.class;
			}
			
		},
		new CdmEnumDataHolder<ReferenceType>(){

			@Override
			public String getName() {
				return "Reference Type";
			}

			@Override
			public Collection<ReferenceType> getElements() {
				return Arrays.asList(ReferenceType.values());
			}

			@Override
			public Class<ReferenceType> getClazz() {
				return ReferenceType.class;
			}
			
		}
	};
	
	private static final int DEFAULT_VISIBLE_ITEMS = 10;

	protected T selection;

	protected List<T> elementTypeList = new ArrayList<T>();

	private Label label;
	protected Combo combo;

	private Class<T> enumType;

	/**
	 * <p>
	 * Constructor for AbstractEnumComboElement.
	 * </p>
	 * 
	 * @param formFactory
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.CdmFormFactory}
	 *            object.
	 * @param parentElement
	 *            a {@link eu.etaxonomy.taxeditor.ui.element.ICdmFormElement}
	 *            object.
	 * @param labelString
	 *            a {@link java.lang.String} object.
	 * @param <T>
	 *            a T object.
	 */
	public EnumComboElement(CdmFormFactory formFactory,
			ICdmFormElement parentElement, Class<T> enumType, int style) {
		super(formFactory, parentElement);
		this.enumType = enumType;
		label = formFactory.createLabel(getLayoutComposite(), getDefaultLabelString());
		addControl(label);

		// create combo
		combo = new Combo(getLayoutComposite(), SWT.BORDER | SWT.READ_ONLY);
		addControl(combo);
		combo.setLayoutData(LayoutConstants.FILL_HORIZONTALLY());
		setVisibleItemCount(DEFAULT_VISIBLE_ITEMS);

		populateTypes();

		combo.addSelectionListener(this);
	}

	private String getDefaultLabelString() {
		if(enumType.equals(ReferenceType.class)){
			return "Reference Type";
		}else if (enumType.equals(NomenclaturalCode.class)){
			return "Nomenclatural Code";
		}
		return "No Label";
	}

	/**
	 * <p>
	 * populateTypes
	 * </p>
	 */
	private void populateTypes(){
		for(T enumElement : getElementsForClass(enumType)){
			elementTypeList.add(enumElement);
			combo.add(((Enum) enumElement).name());
		}
	}

	/**
	 * <p>
	 * Setter for the field <code>selection</code>.
	 * </p>
	 * 
	 * @param selection
	 *            the selection to set
	 */
	public void setSelection(T selection) {
		this.selection = selection;
		combo.select(elementTypeList.indexOf(selection));
	}

	/**
	 * <p>
	 * addSelectionListener
	 * </p>
	 * 
	 * @param listener
	 *            a {@link org.eclipse.swt.events.SelectionListener} object.
	 */
	public void addSelectionListener(SelectionListener listener) {
		combo.addSelectionListener(listener);
	}

	/**
	 * <p>
	 * removeSelectionListener
	 * </p>
	 * 
	 * @param listener
	 *            a {@link org.eclipse.swt.events.SelectionListener} object.
	 */
	public void removeSelectionListener(SelectionListener listener) {
		combo.removeSelectionListener(listener);
	}

	/** {@inheritDoc} */
	public void setSelected(boolean selected) {
		setBackground(selected ? SELECTED : getPersistentBackground());
	}

	/**
	 * <p>
	 * Getter for the field <code>selection</code>.
	 * </p>
	 * 
	 * @return the selection
	 */
	public T getSelection() {
		return selection;
	}

	/** {@inheritDoc} */
	public void setEnabled(boolean enabled) {
		combo.setEnabled(enabled);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt
	 * .events.SelectionEvent)
	 */
	/** {@inheritDoc} */
	public void widgetSelected(SelectionEvent e) {
		selection = elementTypeList.get(combo.getSelectionIndex());
		firePropertyChangeEvent(new CdmPropertyChangeEvent(this, e));
	}

	/** {@inheritDoc} */
	public void setIrrelevant(boolean irrelevant) {
		String colorId = irrelevant ? Resources.COLOR_COMPOSITE_IRRELEVANT
				: Resources.COLOR_COMPOSITE_BACKGROUND;

		Color color = StoreUtil.getColor(colorId);
		combo.setBackground(color);
	}

	/** {@inheritDoc} */
	@Override
	public void setBackground(Color color) {
		label.setBackground(color);
	}

	/** {@inheritDoc} */
	public void widgetDefaultSelected(SelectionEvent e) {
	}
	
	public void setVisibleItemCount(int visibleItems){
		combo.setVisibleItemCount(visibleItems);
	}
	
	private Collection<T> getElementsForClass(Class<T> clazz){
		CdmEnumDataHolder<T> dataHolder = getCdmEnumDataHolderForClass(clazz);
		if (dataHolder != null) {
			return dataHolder.getElements(); 
		}
		return null;
	}
	
	private CdmEnumDataHolder<T> getCdmEnumDataHolderForClass(Class<T> clazz){
		for (CdmEnumDataHolder dataHolder : cdmEnumDataHolders) {
			if (dataHolder.getClazz().equals(clazz)){
				return dataHolder;
			}
		}
		return null;
	}
	
	private interface CdmEnumDataHolder<T> {
		Class<T> getClazz();
		String getName();
		Collection<T> getElements();
	}
}
